param([switch]$Elevated)
function Check-Admin {
$currentUser = New-Object Security.Principal.WindowsPrincipal $([Security.Principal.WindowsIdentity]::GetCurrent())
$currentUser.IsInRole([Security.Principal.WindowsBuiltinRole]::Administrator)
}
if ((Check-Admin) -eq $false)  {
if ($elevated)
{
# could not elevate, quit
}
 
else {
 
Start-Process powershell.exe -Verb RunAs -ArgumentList ('-noprofile -noexit -file "{0}" -elevated' -f ($myinvocation.MyCommand.Definition))
}
exit
}

param (
    [switch]$restart = $FALSE,
    [switch]$norestart = $FALSE,
    [switch]$verbose = $FALSE
 )
 
 $retval = $true

#
#	Uninstall Tricerat Products
#

if($verbose) { Write-Host 'Checking for Tricerat products...' }
if($verbose) { Write-Host }

$applist = Get-WmiObject -class Win32_Product | Where-Object {  $_.Name -like "*Simplify Suite*" -or 
                                                                $_.Name -like "*Simplify License*" -or 
                                                                $_.Name -like "*ScrewDrivers*" }

if($applist -eq $null)
{
    if($verbose) { Write-Host 'No Tricerat products to uninstall' }
}
else
{
   foreach($app in $applist)
   {
       if($verbose) { Write-Host 'Uninstalling ' + $app.Name }
       $errcode = $app.Uninstall().ReturnValue
       if($errcode -ne 0)
       {
           $retval = $false
           if($verbose) { Write-Host 'Failed to uninstall ' + $app.Name + ' with error code: ' + $errcode }
       }
   }
}

# Wait ~5 seconds for additional cleanup (services to be fully removed, etc.).
sleep -Milliseconds 5000

#
#	Delete Services
#

if($verbose) { Write-Host }
if($verbose) { Write-Host 'Checking for Tricerat services...' }
if($verbose) { Write-Host }

$servicelist = Get-WmiObject Win32_Service -Filter "DisplayName LIKE '%Simplify License%' or 
                                                    DisplayName LIKE '%Simplify Lockdown%' or 
                                                    DisplayName LIKE '%Simplify Notification Service%' or
                                                    DisplayName LIKE '%Simplify Profiles%' or
                                                    DisplayName LIKE '%Simplify Stability%' or
                                                    DisplayName LIKE '%ScrewDrivers%'"
			
if($servicelist -eq $null)
{
    if($verbose) { Write-Host 'No Tricerat services to uninstall' }
}
else
{
    foreach($service in $servicelist)
    {
        if($service.State -ne 'Stopped')
        {
            if($verbose) { Write-Host 'Stopping ' + $service.DisplayName }
            $errcode = $service.StopService().ReturnValue
            if($errcode -ne 0)
            {
                $retval = $false
                if($verbose) { Write-Host 'Failed to stop service ' + $service.DisplayName + ' with error code: ' + $errcode }
            }
            else
            {
                # Wait for the service to report that it is stopped.
                $maxRepeat = 20
                $status = "Stopped"
                do 
                {
                    $count = (Get-Service $service.Name | ? {$_.status -eq $status}).count
                    $maxRepeat--
                    sleep -Milliseconds 600
                } until ($count -eq 0 -or $maxRepeat -eq 0)
            }
        }
    
        if($verbose) { Write-Host 'Deleting ' + $service.DisplayName }
        $errcode = $service.delete().ReturnValue
        if($errcode -ne 0 -And $errcode -ne 16) # Error code 16 indicates that the service has already been marked for deletion.
        {
            $retval = $false
            if($verbose) { Write-Host 'Failed to delete ' + $service.DisplayName ' with error code: ' $errcode }
        }
    }
}

#
#	Clean Registry
#
#

if($verbose) { Write-Host }
if($verbose) { Write-Host 'Cleaning up registry keys...' }
if($verbose) { Write-Host }

if($verbose) { Write-Host 'Removing HKCU:\Software\Tricerat' }
if(Test-Path 'HKCU:\Software\Tricerat')
{
    Remove-Item -Path 'HKCU:\Software\Tricerat' -recurse
    sleep -Milliseconds 500

    if(Test-Path 'HKCU:\Software\Tricerat')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to cleanup HKCU.' }
    }
}

if($verbose) { Write-Host 'Removing HKLM:\Software\Tricerat' }
if(Test-Path 'HKLM:\Software\Tricerat')
{
    Remove-Item -Path 'HKLM:\Software\Tricerat' -recurse
    sleep -Milliseconds 500    

    if(Test-Path 'HKLM:\Software\Tricerat')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to cleanup HKLM.' }
    }
}

if(Test-Path 'HKLM:\Software\WOW6432Node\Tricerat')
{
    if($verbose) { Write-Host 'Removing HKLM:\Software\WOW6432Node\Tricerat' }
    Remove-Item -Path 'HKLM:\Software\WOW6432Node\Tricerat' -recurse
    sleep -Milliseconds 500    
    
    if(Test-Path 'HKLM:\Software\WOW6432Node\Tricerat')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to cleanup HKLM.' }
    }
}

# Remove our entries in AppInit data.
if($verbose) { Write-Host 'Cleaning up AppInit registry entry...' }

$appInitKey = 'HKLM:\Software\Microsoft\Windows NT\CurrentVersion\Windows'
$lockRegex = 'Lock..\.dll'
$stabRegex = 'StabAI..\.dll'
$commaRegex = ',?'

# Get the current AppInit_DLLs value.
$currentAppInitValue = (Get-ItemProperty -Path $appInitKey -Name AppInit_DLLs).AppInit_DLLs

# Remove the Lockdown and Stability entries, accounting for comma position.
if([regex]::match($currentAppInitValue, $lockRegex).Index -eq 0) { $lockRegex = $lockRegex + $commaRegex } else { $lockRegex = $commaRegex + $lockRegex }
$newAppInitValue = $currentAppInitValue -replace $lockRegex, ""

if([regex]::match($newAppInitValue, $stabRegex).Index -eq 0) { $stabRegex = $stabRegex + $commaRegex } else { $stabRegex = $commaRegex + $stabRegex }
$newAppInitValue = $newAppInitValue -replace $stabRegex, ""

# Overwrite the AppInit_DLLs value with the entries removed.
Set-ItemProperty -Path $appInitKey -Name AppInit_DLLs -Value $newAppInitValue

# Repeat for Wow6432Node
if(Test-Path 'HKLM:\Software\WOW6432Node\Microsoft\Windows NT\CurrentVersion\Windows')
{
    $appInitKey = 'HKLM:\Software\WOW6432Node\Microsoft\Windows NT\CurrentVersion\Windows'
    $lockRegex = 'Lock..\.dll'
    $stabRegex = 'StabAI..\.dll'

    # Get the current AppInit_DLLs value.
    $currentAppInitValue = (Get-ItemProperty -Path $appInitKey -Name AppInit_DLLs).AppInit_DLLs

    # Remove the Lockdown and Stability entries, accounting for comma position.
    if([regex]::match($currentAppInitValue, $lockRegex).Index -eq 0) { $lockRegex = $lockRegex + $commaRegex } else { $lockRegex = $commaRegex + $lockRegex }
    $newAppInitValue = $currentAppInitValue -replace $lockRegex, ""

    if([regex]::match($newAppInitValue, $stabRegex).Index -eq 0) { $stabRegex = $stabRegex + $commaRegex } else { $stabRegex = $commaRegex + $stabRegex }
    $newAppInitValue = $newAppInitValue -replace $stabRegex, ""

    # Overwrite the AppInit_DLLs value with the entries removed.
    Set-ItemProperty -Path $appinitkey -Name AppInit_DLLs -Value $newAppInitValue
}

#
#	Remove Tricerat Directory
#

if($verbose) { Write-Host }
if($verbose) { Write-Host 'Deleting Tricerat program directory...' }

if(Test-Path $env:ProgramFiles'\Tricerat')
{
    Remove-Item $env:ProgramFiles'\Tricerat' -recurse -force
    sleep -Milliseconds 500

    if(Test-Path $env:ProgramFiles'\Tricerat')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to cleanup Program Files.' }
    }
}

if(${Env:ProgramFiles(x86)})
{
    if($verbose) { Write-Host }
    if($verbose) { Write-Host 'Deleting Tricerat program directory x86...' }
    if(Test-Path ${Env:ProgramFiles(x86)}'\Tricerat')
    {
        Remove-Item ${Env:ProgramFiles(x86)}'\Tricerat' -recurse -force
        sleep -Milliseconds 500

        if(Test-Path ${Env:ProgramFiles(x86)}'\Tricerat')
        {
            $retval = $false
            if($verbose) { Write-Host 'Failed to cleanup Program Files(x86).' }
        }
    }
}
#
#	Cleanup Windows Folder
#

if($verbose) { Write-Host }
if($verbose) { Write-Host 'Cleaning up the Windows directory...' }
if($verbose) { Write-Host }

# Stop the print spooler so we can cleanup files.
if($verbose) { Write-Host 'Stopping the Print Spooler...' }
Stop-Service Spooler

# Wait for the service to report that it is stopped.
$services = "Spooler"
$maxRepeat = 20
$status = "Stopped"
do 
{
    $count = (Get-Service $services | ? {$_.status -eq $status}).count
    $maxRepeat--
    sleep -Milliseconds 600
} until ($count -eq 0 -or $maxRepeat -eq 0)

if($verbose) { Write-Host 'Removing Driver and DriverUI files...' }

# Remove our files from the spool directory (x64).
if(Test-Path $env:windir'\System32\spool\drivers\x64\sd4*')
{
    Remove-Item $env:windir'\System32\spool\drivers\x64\sd4*'
    
    if(Test-Path $env:windir'\System32\spool\drivers\x64\sd4*')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to cleanup 64-bit spool drivers.' }
    }
}

if(Test-Path $env:windir'\System32\spool\drivers\x64\3\sd4*')
{
    Remove-Item $env:windir'\System32\spool\drivers\x64\3\sd4*'
    
    if(Test-Path $env:windir'\System32\spool\drivers\x64\3\sd4*')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to cleanup 64-bit spool drivers in 3.' }
    }
}

# Remove our files from the spool directory (x86).
if(Test-Path $env:windir'\System32\spool\drivers\W32X86\sd4*')
{
    Remove-Item $env:windir'\System32\spool\drivers\W32X86\sd4*'
    
    if(Test-Path $env:windir'\System32\spool\drivers\W32X86\sd4*')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to cleanup 32-bit spool drivers.' }
    }
}

if(Test-Path $env:windir'\System32\spool\drivers\W32X86\3\sd4*')
{
    Remove-Item $env:windir'\System32\spool\drivers\W32X86\3\sd4*'
    
    if(Test-Path $env:windir'\System32\spool\drivers\W32X86\3\sd4*')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to cleanup 32-bit spool drivers in 3.' }
    }
}

# Start the print spooler when we're done.
if($verbose) { Write-Host 'Starting the Print Spooler...' }
Start-Service Spooler

# Remove TriceratUserInit.
if($verbose) { Write-Host 'Removing TriceratUserInit...' }
if(Test-Path $env:windir'\System32\TriceratUserInit.exe')
{
    Remove-Item $env:windir'\System32\TriceratUserInit.exe'
    
    if(Test-Path $env:windir'\System32\TriceratUserInit.exe')
    {
        $retval = $false
        if($verbose) { Write-Host 'Could not remove TriceratUserInit.' }
    }
}

# Remove lockdown and stability files.
if($verbose) { Write-Host 'Removing Lockdown and Stability files...' }

if(Test-Path $env:windir'\System32\StabAI*')
{
    Remove-Item $env:windir'\System32\StabAI*'
    
    if(Test-Path $env:windir'\System32\StabAI*')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to remove Stability files from Sys32.' }
    }
}

if(Test-Path $env:windir'\SysWOW64\StabAI*')
{
    Remove-Item $env:windir'\SysWOW64\StabAI*'
    
    if(Test-Path $env:windir'\SysWOW64\StabAI*')
    {
        $retval = $false
        if($verbose) { Write-Host 'Faild to remove Stability files from SysWOW64.' }
    }
}

if(Test-Path $env:windir'\System32\Lock*')
{
    Remove-Item $env:windir'\System32\Lock*'
    
    if(Test-Path $env:windir'\System32\Lock*')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to remove Lockdown files from Sys32.' }
    }
}

if(Test-Path $env:windir'\SysWOW64\Lock*')
{
    Remove-Item $env:windir'\SysWOW64\Lock*'
    
    if(Test-Path $env:windir'\SysWOW64\Lock*')
    {
        $retval = $false
        if($verbose) { Write-Host 'Failed to remove Lockdown files from SysWOW64.' }
    }
}

# Indicate success or failure before rebooting.
if($verbose) { Write-Host }
if($retval)
{
    Write-Host 'Cleanup completed successfully.'
    $retcode = 0
    
    #
    #	Prompt to Restart Server if Successful
    #

    if($verbose) { Write-Host }

    # If neither flag was set, prompt the user.
    if(!$restart -And !$norestart)
    {
        $restartprompt = Read-Host -Prompt 'Server restart required. Restart now? (y/n) '
        if($restartprompt -eq 'y' -Or $restartprompt -eq 'Y')
        {
            $restart = $TRUE        
        }
    }

    # Restart if restart flag was specified or user requested restart.
    if($restart)
    {
        Restart-Computer
    }
}
else
{
    Write-Host 'Cleanup failed. Manual cleanup required.'
    $retcode = -1
}

exit $retcode